mod api_operation;
mod sse_invoke_method;
mod custom_jwt;
use crate::sse_invoke_method::sse_invoke::constant_value::{API_KEY};

use std::env;
use std::fs;
use std::io::Read;

#[derive(Debug)]
pub struct C2RustGLM {
    chatglm_response: String,
}

impl C2RustGLM {
    pub async fn new() -> Self {
        C2RustGLM {
            chatglm_response: String::new(),
        }
    }

    pub async fn sse_invoke_calling(&mut self, api_key: &str, user_input: &str) {
        let mut require_calling = "SSE".to_string();
        let mut input = String::new();

        
        let api_key_instance = api_operation::APIKeys::get_instance(api_key);
        let jwt_creator =
            custom_jwt::CustomJwt::new(api_key_instance.get_user_id(), api_key_instance.get_user_secret());
        let jwt = jwt_creator.create_jwt();

        let jwt_to_verify = jwt.clone();
        let is_valid = jwt_creator.verify_jwt(&jwt_to_verify);
    
        let sse_call = sse_invoke_method::ReceiveSSEInvokeModelOnlyText::new(&jwt, &user_input.trim());

        self.chatglm_response = match sse_call.await.get_response_message() {
            Some(message) => message.to_string(), // Return the message as String
            None => "Error: Unable to get SSE response.".to_string(),
        }
    }

    pub fn get_ai_response(&self) -> String {
        self.chatglm_response.clone()
    }
}

#[tokio::main]
async fn main()
{
    let args: Vec<String> = env::args().collect(); // 获取命令行参数
    
    if args.len() < 2 {
        println!("Usage: {} <cpp_file_path>", &args[0]); // 输出正确的使用方法
        return;
    }
    
    let file_path = &args[1]; // 第二个参数为文件路径
    let mut content = String::new();
    let mut prompt = "Convert the C++ code to Rust: ".to_string();

    match fs::File::open(file_path) {
        Ok(mut file) => {
            match file.read_to_string(&mut content) {
                Ok(_) => {
                    prompt += content.as_str(); // 合成最终Prompt内容
                },
                Err(err) => {
                    eprintln!("Failed to read the file: {:?}", err); // 错误处理
                }
            };
        },
        Err(err) => {
            eprintln!("Failed to open the file: {:?}", err); // 错误处理
        }
    };

    let mut c_2_rust_glm = C2RustGLM::new().await;
    
    c_2_rust_glm.sse_invoke_calling(API_KEY, prompt.as_str()).await;
    println!("{}", c_2_rust_glm.get_ai_response());
}